"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const apputils_1 = require("@jupyterlab/apputils");
require("../style/upload.css");
exports.STRATEGY = {
    continue: 'continue',
    create: 'create'
};
class NeptuneUploadButton extends apputils_1.ReactWidget {
    constructor(content, connection) {
        super();
        this.content = content;
        this.connection = connection;
    }
    render() {
        return (React.createElement(UploadButton, { connection: this.connection, content: this.content }));
    }
}
exports.NeptuneUploadButton = NeptuneUploadButton;
const TIMEOUT_TIME = 4000;
class UploadButton extends React.Component {
    constructor(props) {
        super(props);
        this.getErrorText = (status) => {
            switch (status) {
                case 422: {
                    return (React.createElement("div", { className: "n-upload-notice" }, "Storage limit has been reached. Checkpoint can't be uploaded."));
                }
                default: {
                    return (React.createElement("div", { className: "n-upload-notice" }, "An error occurred during notebook upload. Please try again."));
                }
            }
        };
        this.getNotebookURI = () => {
            if (!this.state.notebook) {
                return null;
            }
            const { id, name, lastCheckpointId, projectId, } = this.state.notebook;
            const apiAddress = this.props.connection.getApiAddress();
            const project = this.state.projects.find(project => project.id === projectId);
            return `${apiAddress}\/${project.organizationName}/${project.name}\/n\/${name}\-${id}\/${lastCheckpointId}`;
        };
        this.validateUpload = () => {
            const { connection, content } = this.props;
            content
                .validateMetadata()
                .then(() => connection.validate())
                .then(() => {
                return connection.getNotebook().then(notebook => {
                    return connection.getUsername().then(username => {
                        if (notebook.owner !== username) {
                            return Promise.reject('Notebook has different owner');
                        }
                    });
                });
            })
                .then(() => this.setState({ isUploadAvailable: true }))
                .catch(() => this.setState({ isUploadAvailable: false }));
        };
        this.updateResolveStrategy = (strategy) => {
            this.setState({
                conflictResolveStrategy: strategy
            });
        };
        this.uploadNotebook = () => {
            const { connection, content } = this.props;
            const path = content.getNotebookPath();
            this.setLoading();
            connection
                .getNotebook()
                .then(notebook => {
                if (!notebook.id) {
                    return this.createNotebook();
                }
                this.setState({ notebook });
                if (notebook.path !== path) {
                    return apputils_1.showDialog({
                        body: React.createElement(UploadDialog, { onUpdateResolveStrategyChange: this.updateResolveStrategy }),
                        buttons: [
                            apputils_1.Dialog.createButton({ label: 'Cancel', accept: false }),
                            apputils_1.Dialog.createButton({ label: 'Apply', accept: true })
                        ]
                    })
                        .then(result => {
                        if (result.button.accept) {
                            if (this.state.conflictResolveStrategy === exports.STRATEGY.create) {
                                return this.createNotebook();
                            }
                        }
                        else {
                            return Promise.reject();
                        }
                    });
                }
            })
                .then(content.saveContent)
                .then(content.getNotebookContent)
                .then(notebookContent => connection.createCheckpoint(path, notebookContent))
                .then(notebook => this.setState({ notebook }))
                .then(() => connection.listProjects().then((entries) => this.setState({ projects: entries })))
                .then(() => this.setUploaded())
                .catch((error) => this.setRejected(error));
        };
        this.createNotebook = () => {
            const { connection, content, } = this.props;
            return connection
                .createNotebook(content.getNotebookPath())
                .then(notebookId => connection.updateParams({ notebookId }));
        };
        this.setLoading = () => {
            this.resetUploadState();
            this.setState({ uploadStatus: 'loading' });
        };
        this.setUploaded = () => {
            this.resetUploadState();
            this.setState({ uploadStatus: 'success' });
            this.timeout = setTimeout(() => this.resetUploadState(), TIMEOUT_TIME);
        };
        this.setRejected = (status) => {
            this.resetUploadState();
            this.setState({ uploadStatus: 'fail', errorStatus: status });
            this.timeout = setTimeout(() => this.resetUploadState(), TIMEOUT_TIME);
        };
        this.state = {
            conflictResolveStrategy: exports.STRATEGY.continue,
            isUploadAvailable: false,
        };
        this.props.connection.paramsChanged.connect(() => {
            this.validateUpload();
        });
        this.validateUpload();
    }
    render() {
        let className = 'n-UploadButton';
        let glyph = 'fa-arrow-circle-up';
        switch (this.state.uploadStatus) {
            case 'loading':
                glyph = 'fas fa-spinner fa-spin fa-w-14';
                break;
            case 'success':
                className += ' n-UploadButton--success';
                break;
            case 'fail':
                className += ' n-UploadButton--failed';
                glyph = 'fa-times-circle';
                break;
        }
        return (React.createElement("div", { className: "n-Button-Wrapper" },
            React.createElement(apputils_1.ToolbarButtonComponent, { className: className, iconClassName: 'fa ' + glyph, label: 'Upload', onClick: this.uploadNotebook, tooltip: 'Upload to Neptune', enabled: this.state.isUploadAvailable }),
            (this.state.uploadStatus === 'success') && (React.createElement("div", { className: "n-upload-notice" },
                "Successfully uploaded! To see notebook in Neptune, go to ",
                React.createElement("a", { href: this.getNotebookURI(), target: "_blank", rel: "noopener noreferrer" }, "link"),
                ".")),
            (this.state.uploadStatus === 'fail') && this.getErrorText(this.state.errorStatus)));
    }
    resetUploadState() {
        clearTimeout(this.timeout);
        this.setState({ uploadStatus: null, errorStatus: null });
    }
    ;
}
class UploadDialog extends React.Component {
    constructor() {
        super(...arguments);
        this.updateResolveStrategy = (event) => {
            this.props.onUpdateResolveStrategyChange(event.target.value);
        };
    }
    render() {
        const text = this.props.header || 'Notebook name has changed.';
        return (React.createElement("div", { className: "n-UploadDialog" },
            React.createElement("p", { className: "n-UploadDialog__note" },
                text,
                " How would you like to continue?"),
            React.createElement("label", { className: "n-UploadDialog__option-label" },
                React.createElement("input", { className: "n-UploadDialog__option", type: "radio", name: "strategy", value: exports.STRATEGY.continue, checked: true, onChange: this.updateResolveStrategy }),
                "Append checkpoint to this notebook"),
            React.createElement("label", { className: "n-UploadDialog__option-label" },
                React.createElement("input", { className: "n-UploadDialog__option", type: "radio", name: "strategy", value: exports.STRATEGY.create, onChange: this.updateResolveStrategy }),
                "Create new notebook in Neptune")));
    }
}
exports.UploadDialog = UploadDialog;
//# sourceMappingURL=upload.js.map